/*
 * fatchk - print fat chain
 *   Copyright (C)1999 Tachibana Eriko
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/dos.h>
#include <sys/scsi.h>
#include <sys/xglob.h>


/* File scope variables */
static const char program[] = "fatchk";
static const char version[] = "1.00";
static const char date[] = "1999/11/14";
static const char author[] = "Ԃq";

static char* argv0;


/* File scope function */
static int
get_top_blk (int unit_no, int sec_size, int scsi_id, int blk_size)
{
  unsigned char buf[blk_size];
  int b;

  /* ubNTCY 256<<s ̌`ɕϊ */
  for (b = 3; b >= 0; --b)
    if (blk_size == (256 << b))
      break;
  if (b < 0)
    return -1;

  /* 擪ZN^ǂݍ */
  if (_scsi_readext (0, 1, scsi_id, b, buf) < 0)
    return -1;

  /* X680x0 */
  if (memcmp ("X68SCSI1", buf, 8) == 0)
    {
      int i;

      /* ItZbg 2048 oCg̃ZN^ǂݍ */
      if (_scsi_readext (8 >> b, 1, scsi_id, b, buf) < 0)
	return -1;
      if (memcmp ("X68K", buf, 4) != 0)
	return -1;

      for (i = 1; i < 16; ++i)
	{
	  if (memcmp ("Human68k", &buf[i * 16 + 0], 8) != 0)
	    continue;			/*  Human68k p[eBV */
	  if (buf[i * 16 + 8] & 1)
	    continue;			/* ANeBuȃp[eBV */

	  if (--unit_no < 0)
	    {
	      int r = (*(int*) &buf[i * 16 + 8]) & 0x00ffffff;

	      return (r << (3 - b)) >> 1;
	    }
	}

      return -1;
    }

#if 0	      				/* Ή */
  /* IBM-FDISK */
  else if ((buf[0] == 0xb8 || buf[0] == 0xfa || buf[0] == 0x00)
	&& buf[sec_size - 2] == 0x55 && buf[sec_size - 1] == 0xaa)
    {
      return -1;
    }
#endif

  /* IBM-PC Super-Floppy */
  else if (buf[0] == 0xeb
	&& buf[sec_size - 2] == 0x55 && buf[sec_size - 1] == 0xaa)
    {
      return (unit_no == 0) ? 0 : -1;
    }

  /* ZIP Iomega */
  else if (memcmp ("IPAR", buf + 2, 4) == 0)
    {
      return (unit_no == 0) ? 0 : -1;
    }

  return -1;
}


/* File scope function */
static void
fat_chk (const char* filename, int scsi)
{
  static unsigned short buf[16*1024];
  unsigned int* p;
  int rc;

  if ((rc = _dos_fatchk2 (filename, buf, sizeof buf)) < 0)
    {
      errno = _errcnv (rc);
      perror (filename);
      return;
    }

  if (scsi)
    {
      struct _readcap cap;
      struct _dpbptr dpb;
      int unit_no, sec_size, scsi_id, blk_size, top_blk;
      int id;

      if ((rc = _dos_getdpb (*buf, &dpb)) < 0
       || (rc = _dos_ioctrlrd (*buf, (char*) &id, 4)) < 0)
	{
	  errno = _errcnv (rc);
	  perror (filename);
	  return;
	}
      unit_no = dpb.unit;
      sec_size = dpb.byte;
      scsi_id = id;

      if ((rc = _scsi_readcap (scsi_id, &cap)) < 0
       || (top_blk = get_top_blk (unit_no, sec_size, scsi_id, blk_size = cap.size)) < 0)
	{
	  eprintf ("%s: Unknown disk format\n", filename);
	  return;
	}

      printf ("%s\t(%c: LUN=%d ID=%d UNIT=%d)", filename, *buf + 'A' - 1,
			scsi_id >> 16, scsi_id & 0xffff, unit_no);
      for (p = (unsigned int*) &buf[1]; *p;)
	{
	  int sec = *p++ * sec_size / blk_size + top_blk;
	  int len = *p++ * sec_size / blk_size;

	  printf (" $%x,%d", sec, len);
	}
    }
  else
    {
      printf ("%s\t(%c:)", filename, *buf + 'A' - 1);
      for (p = (unsigned int*) &buf[1]; *p;)
	{
	  int sec = *p++;
	  int len = *p++;

	  printf (" $%x,%d", sec, len);
	}
    }
  printf ("\n");
}


/* File scope function */
static void
print_usage (void)
{
  printf ("%s version %s  %s %s.\n", program, version, date, author);
  printf ("usage: %s [option] [--] [filename] ...\n", argv0);
#if 0
  printf ("option:\n"
	  "\t-h\tgp@\\n");
#endif
}

/* File scope function */
static void
print_version (void)
{
  printf ("%s version %s\n", program, version);
}


/* Function */
int
main (int argc, char* argv[])
{
  int i;
  int count = 0;
  int no_opt = 0;
  int scsi = 0;

  argv0 = argv[0];

  for (i = 1; i < argc; ++i)
    {
      char* p = argv[i];

      if (*p == '\0')
	continue;

      if (*p++ != '-' || *p == '\0' || no_opt)
	{
	  ++count;
	  fat_chk (--p, scsi);
	}

      else if (p[0] == '-' && p[1] == '\0')
	no_opt = 1;			/* -- */

      else if (p[0] == 's' && p[1] == '\0')
	scsi = 1;			/* -s */

      else if (*p == '?' || *p == 'h' || *p == 'H' || strcmp (p, "-help") == 0)
	{
	  print_usage ();		/* gp@\ */
	  return EXIT_SUCCESS;
	}

      else if (*p == 'v' || *p == 'V' || strcmp (p, "-version") == 0)
	{
	  print_version ();		/* o[W\ */
	  return EXIT_SUCCESS;
	}

      else
	{
	  print_usage ();		/* sȈ */
	  return EXIT_FAILURE;
	}
    }

  if (count == 0)
    {
      print_usage ();			/* ȗ */
      return EXIT_FAILURE;
    }

  return EXIT_SUCCESS;
}

/* EOF */
