/* dos_files.c - extract wildcard
 * Copyright (C) 2000 Tachibana Eriko
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <sys/dos.h>

extern unsigned char* mbsrchr (unsigned char*, int);

static struct _filbuf files_buf;
static char filename[PATH_MAX + 1];
static char* filenamep;
static int rc;

char*
files_start (char* wild)
{
  char* p = strcpy (filename, wild);
  char* n;
  char c;

  if (p[0] != '\0' && p[1] == ':')
    p += 2;
  while ((n = mbsrchr (p, '/')) != NULL || (n = mbsrchr (p, '\\')) != NULL)
    p = n + 1;
  filenamep = p;

  while ((c = *p++) != '\0')
    {
      if (c == '?' || c == '*')
	break;
    }
  if (c == '\0')
    {
      rc = -1;
      return filename;
    }

  if ((rc = _dos_files (&files_buf, filename, 0xff)) < 0)
    return filename;

  strcpy (filenamep, files_buf.name);
  return filename;
}

char*
files_next (void)
{
  if (rc < 0 || (rc = _dos_nfiles (&files_buf)) < 0)
    return NULL;

  strcpy (filenamep, files_buf.name);
  return filename;
}


#ifdef TEST
int
main (int argc, char* argv[])
{
  int i;

  for (i = 1; i < argc; ++i)
    {
      char* f;

      for (f = files_start (argv[i]); f; f = files_next ())
	puts (f);
    }

  return 0;
}
#endif

/* EOF */
